/*
 * Copyright (c) 2015 Goldman Sachs.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * and Eclipse Distribution License v. 1.0 which accompany this distribution.
 * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
 * and the Eclipse Distribution License is available at
 * http://www.eclipse.org/org/documents/edl-v10.php.
 */

package org.eclipse.collections.impl.bag.mutable.primitive;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.NoSuchElementException;

import org.eclipse.collections.api.LongIterable;
import org.eclipse.collections.api.bag.MutableBag;
import org.eclipse.collections.api.bag.primitive.LongBag;
import org.eclipse.collections.api.bag.primitive.ImmutableLongBag;
import org.eclipse.collections.api.bag.primitive.MutableLongBag;
import org.eclipse.collections.api.block.function.primitive.LongToObjectFunction;
import org.eclipse.collections.api.block.function.primitive.ObjectLongToObjectFunction;
import org.eclipse.collections.api.block.function.primitive.IntToIntFunction;
import org.eclipse.collections.api.block.predicate.primitive.LongPredicate;
import org.eclipse.collections.api.block.procedure.primitive.LongProcedure;
import org.eclipse.collections.api.block.procedure.primitive.LongIntProcedure;
import org.eclipse.collections.api.iterator.LongIterator;
import org.eclipse.collections.api.iterator.MutableLongIterator;
import org.eclipse.collections.api.set.primitive.LongSet;
import org.eclipse.collections.impl.Counter;
import org.eclipse.collections.impl.bag.mutable.HashBag;
import org.eclipse.collections.impl.block.factory.primitive.IntToIntFunctions;
import org.eclipse.collections.impl.factory.primitive.LongBags;
import org.eclipse.collections.impl.map.mutable.primitive.LongIntHashMap;
import org.eclipse.collections.impl.primitive.AbstractLongIterable;
import org.eclipse.collections.impl.set.mutable.primitive.LongHashSet;
import net.jcip.annotations.NotThreadSafe;

/**
 * LongHashBag is similar to {@link HashBag}, and is memory-optimized for long primitives.
 * This file was automatically generated from template file primitiveHashBag.stg.
 *
 * @since 3.0.
 */
@NotThreadSafe
public final class LongHashBag extends AbstractLongIterable implements MutableLongBag, Externalizable
{
    private static final long serialVersionUID = 1L;

    private LongIntHashMap items;
    private int size;

    public LongHashBag()
    {
        this.items = new LongIntHashMap();
    }

    public LongHashBag(int size)
    {
        this.items = new LongIntHashMap(size);
    }

    public LongHashBag(LongIterable iterable)
    {
        this();
        this.addAll(iterable);
    }

    public LongHashBag(long... elements)
    {
        this();
        this.addAll(elements);
    }

    public LongHashBag(LongHashBag bag)
    {
        this.items = new LongIntHashMap(bag.sizeDistinct());
        this.addAll(bag);
    }

    public static LongHashBag newBag(int size)
    {
        return new LongHashBag(size);
    }

    public static LongHashBag newBagWith(long... source)
    {
        return new LongHashBag(source);
    }

    public static LongHashBag newBag(LongIterable source)
    {
        if (source instanceof LongHashBag)
        {
            return new LongHashBag((LongHashBag) source);
        }
        return new LongHashBag(source);
    }

    public static LongHashBag newBag(LongBag source)
    {
        return new LongHashBag(source);
    }

    @Override
    public boolean isEmpty()
    {
        return this.items.isEmpty();
    }

    @Override
    public boolean notEmpty()
    {
        return this.items.notEmpty();
    }

    @Override
    public int size()
    {
        return this.size;
    }

    public int sizeDistinct()
    {
        return this.items.size();
    }

    public void clear()
    {
        this.items.clear();
        this.size = 0;
    }

    public LongHashBag with(long element)
    {
        this.add(element);
        return this;
    }

    public LongHashBag with(long element1, long element2)
    {
        this.add(element1);
        this.add(element2);
        return this;
    }

    public LongHashBag with(long element1, long element2, long element3)
    {
        this.add(element1);
        this.add(element2);
        this.add(element3);
        return this;
    }

    public LongHashBag withAll(LongIterable iterable)
    {
        this.addAll(iterable);
        return this;
    }

    public LongHashBag without(long element)
    {
        this.remove(element);
        return this;
    }

    public LongHashBag withoutAll(LongIterable iterable)
    {
        this.removeAll(iterable);
        return this;
    }

    public boolean contains(long value)
    {
        return this.items.containsKey(value);
    }

    public int occurrencesOf(long item)
    {
        return this.items.get(item);
    }

    public void forEachWithOccurrences(LongIntProcedure procedure)
    {
        this.items.forEachKeyValue(procedure);
    }

    public boolean add(long item)
    {
        this.items.updateValue(item, 0, IntToIntFunctions.increment());
        this.size++;
        return true;
    }

    public boolean remove(long item)
    {
        int newValue = this.items.updateValue(item, 0, IntToIntFunctions.decrement());
        if (newValue <= 0)
        {
            this.items.removeKey(item);
            if (newValue == 0)
            {
                this.size--;
            }
            return newValue == 0;
        }
        this.size--;
        return true;
    }

    public boolean addAll(long... source)
    {
        if (source.length == 0)
        {
            return false;
        }

        for (long each : source)
        {
            this.add(each);
        }
        return true;
    }

    public boolean addAll(LongIterable source)
    {
        if (source.isEmpty())
        {
            return false;
        }
        if (source instanceof LongBag)
        {
            LongBag otherBag = (LongBag) source;
            otherBag.forEachWithOccurrences(new LongIntProcedure()
            {
                public void value(long each, int occurrences)
                {
                    LongHashBag.this.addOccurrences(each, occurrences);
                }
            });
        }
        else
        {
            LongIterator iterator = source.longIterator();
            while (iterator.hasNext())
            {
                long each = iterator.next();
                this.add(each);
            }
        }
        return true;
    }

    public boolean removeAll(long... source)
    {
        if (source.length == 0)
        {
            return false;
        }
        int oldSize = this.size();
        for (long each : source)
        {
            int occurrences = this.items.removeKeyIfAbsent(each, 0);
            this.size -= occurrences;
        }
        return this.size() != oldSize;
    }

    public boolean removeAll(LongIterable source)
    {
        if (source.isEmpty())
        {
            return false;
        }
        int oldSize = this.size();
        if (source instanceof LongBag)
        {
            LongBag otherBag = (LongBag) source;
            otherBag.forEachWithOccurrences(new LongIntProcedure()
            {
                public void value(long each, int occurrences)
                {
                    int oldOccurrences = LongHashBag.this.items.removeKeyIfAbsent(each, 0);
                    LongHashBag.this.size -= oldOccurrences;
                }
            });
        }
        else
        {
            LongIterator iterator = source.longIterator();
            while (iterator.hasNext())
            {
                long each = iterator.next();
                int occurrences = this.items.removeKeyIfAbsent(each, 0);
                this.size -= occurrences;
            }
        }
        return this.size() != oldSize;
    }

    public boolean retainAll(LongIterable source)
    {
        int oldSize = this.size();
        final LongSet sourceSet = source instanceof LongSet ? (LongSet) source : source.toSet();
        LongHashBag retained = this.select(new LongPredicate()
        {
            public boolean accept(long key)
            {
                return sourceSet.contains(key);
            }
        });
        if (retained.size() != oldSize)
        {
            this.items = retained.items;
            this.size = retained.size;
            return true;
        }
        return false;
    }

    public boolean retainAll(long... source)
    {
        return this.retainAll(LongHashSet.newSetWith(source));
    }

    public void addOccurrences(long item, final int occurrences)
    {
        if (occurrences < 0)
        {
            throw new IllegalArgumentException("Cannot add a negative number of occurrences");
        }
        if (occurrences > 0)
        {
            this.items.updateValue(item, 0, new IntToIntFunction()
            {
                public int valueOf(int intParameter)
                {
                    return intParameter + occurrences;
                }
            });
            this.size += occurrences;
        }
    }

    public boolean removeOccurrences(long item, final int occurrences)
    {
        if (occurrences < 0)
        {
            throw new IllegalArgumentException("Cannot remove a negative number of occurrences");
        }

        if (occurrences == 0)
        {
            return false;
        }

        int newValue = this.items.updateValue(item, 0, new IntToIntFunction()
        {
            public int valueOf(int intParameter)
            {
                return intParameter - occurrences;
            }
        });

        if (newValue <= 0)
        {
            this.size -= occurrences - newValue;
            this.items.removeKey(item);
            return newValue + occurrences != 0;
        }

        this.size -= occurrences;
        return true;
    }

    public void forEach(LongProcedure procedure)
    {
        this.each(procedure);
    }

    /**
     * @since 7.0.
     */
    public void each(final LongProcedure procedure)
    {
        this.items.forEachKeyValue(new LongIntProcedure()
        {
            public void value(long key, int occurrences)
            {
                for (int i = 0; i < occurrences; i++)
                {
                    procedure.value(key);
                }
            }
        });
    }

    public LongHashBag select(final LongPredicate predicate)
    {
        final LongHashBag result = new LongHashBag();
        this.forEachWithOccurrences(new LongIntProcedure()
        {
            public void value(long each, int occurrences)
            {
                if (predicate.accept(each))
                {
                    result.addOccurrences(each, occurrences);
                }
            }
        });
        return result;
    }

    public LongHashBag reject(final LongPredicate predicate)
    {
        final LongHashBag result = new LongHashBag();
        this.forEachWithOccurrences(new LongIntProcedure()
        {
            public void value(long each, int occurrences)
            {
                if (!predicate.accept(each))
                {
                    result.addOccurrences(each, occurrences);
                }
            }
        });
        return result;
    }

    public <T> T injectInto(T injectedValue, ObjectLongToObjectFunction<? super T, ? extends T> function)
    {
        T result = injectedValue;
        LongIterator it = this.longIterator();
        while (it.hasNext())
        {
            result = function.valueOf(result, it.next());
        }
        return result;
    }

    @Override
    public boolean equals(Object otherBag)
    {
        if (otherBag == this)
        {
            return true;
        }
        if (!(otherBag instanceof LongBag))
        {
            return false;
        }
        final LongBag bag = (LongBag) otherBag;
        if (this.sizeDistinct() != bag.sizeDistinct())
        {
            return false;
        }

        return this.items.keysView().allSatisfy(new LongPredicate()
        {
            public boolean accept(long key)
            {
                return LongHashBag.this.occurrencesOf(key) == bag.occurrencesOf(key);
            }
        });
    }

    @Override
    public int hashCode()
    {
        final Counter result = new Counter();
        this.forEachWithOccurrences(new LongIntProcedure()
        {
            public void value(long eachItem, int occurrences)
            {
                result.add((int) (eachItem ^ eachItem >>> 32) ^ occurrences);
            }
        });
        return result.getCount();
    }

    public void appendString(
            final Appendable appendable,
            String start,
            final String separator,
            String end)
    {
        final boolean[] firstItem = {true};
        try
        {
            appendable.append(start);
            this.items.forEachKeyValue(new LongIntProcedure()
            {
                public void value(long each, int occurrences)
                {
                    try
                    {
                        for (int i = 0; i < occurrences; i++)
                        {
                            if (!firstItem[0])
                            {
                                appendable.append(separator);
                            }
                            appendable.append(String.valueOf(each));
                            firstItem[0] = false;
                        }
                    }
                    catch (IOException e)
                    {
                        throw new RuntimeException(e);
                    }
                }
            });
            appendable.append(end);
        }
        catch (IOException e)
        {
            throw new RuntimeException(e);
        }
    }

    public int count(final LongPredicate predicate)
    {
        final Counter result = new Counter();
        this.forEachWithOccurrences(new LongIntProcedure()
        {
            public void value(long each, int occurrences)
            {
                if (predicate.accept(each))
                {
                    result.add(occurrences);
                }
            }
        });
        return result.getCount();
    }

    public boolean anySatisfy(LongPredicate predicate)
    {
        return this.items.keysView().anySatisfy(predicate);
    }

    public boolean allSatisfy(LongPredicate predicate)
    {
        return this.items.keysView().allSatisfy(predicate);
    }

    public boolean noneSatisfy(LongPredicate predicate)
    {
        return this.items.keysView().noneSatisfy(predicate);
    }

    public long detectIfNone(LongPredicate predicate, long ifNone)
    {
        return this.items.keysView().detectIfNone(predicate, ifNone);
    }

    public <V> MutableBag<V> collect(final LongToObjectFunction<? extends V> function)
    {
        final HashBag<V> result = HashBag.newBag(this.items.size());
        this.forEachWithOccurrences(new LongIntProcedure()
        {
            public void value(long each, int occurrences)
            {
                result.addOccurrences(function.valueOf(each), occurrences);
            }
        });
        return result;
    }

    public long max()
    {
        if (this.isEmpty())
        {
            throw new NoSuchElementException();
        }
        return this.items.keysView().max();
    }

    public long min()
    {
        if (this.isEmpty())
        {
            throw new NoSuchElementException();
        }
        return this.items.keysView().min();
    }

    public long sum()
    {
        final long[] result = {0L};
        this.forEachWithOccurrences(new LongIntProcedure()
        {
            public void value(long each, int occurrences)
            {
                result[0] += (long) each * occurrences;
            }
        });
        return result[0];
    }

    public long[] toArray()
    {
        final long[] array = new long[this.size()];
        final int[] index = {0};

        this.forEachWithOccurrences(new LongIntProcedure()
        {
            public void value(long each, int occurrences)
            {
                for (int i = 0; i < occurrences; i++)
                {
                    array[index[0]] = each;
                    index[0]++;
                }
            }
        });
        return array;
    }

    public MutableLongBag asUnmodifiable()
    {
        return new UnmodifiableLongBag(this);
    }

    public MutableLongBag asSynchronized()
    {
        return new SynchronizedLongBag(this);
    }

    public ImmutableLongBag toImmutable()
    {
        return LongBags.immutable.withAll(this);
    }

    public MutableLongIterator longIterator()
    {
        return new InternalIterator();
    }

    public void writeExternal(final ObjectOutput out) throws IOException
    {
        out.writeInt(this.items.size());
        try
        {
            this.items.forEachKeyValue(new LongIntProcedure()
            {
                public void value(long each, int occurrences)
                {
                    try
                    {
                        out.writeLong(each);
                        out.writeInt(occurrences);
                    }
                    catch (IOException e)
                    {
                        throw new RuntimeException(e);
                    }
                }
            });
        }
        catch (RuntimeException e)
        {
            if (e.getCause() instanceof IOException)
            {
                throw (IOException) e.getCause();
            }
            throw e;
        }
    }

    public void readExternal(ObjectInput in) throws IOException
    {
        int size = in.readInt();
        this.items = new LongIntHashMap(size);
        for (int i = 0; i < size; i++)
        {
            this.addOccurrences(in.readLong(), in.readInt());
        }
    }

    private class InternalIterator implements MutableLongIterator
    {
        private final MutableLongIterator longIterator = LongHashBag.this.items.keySet().longIterator();

        private long currentItem;
        private int occurrences;
        private boolean canRemove;

        public boolean hasNext()
        {
            return this.occurrences > 0 || this.longIterator.hasNext();
        }

        public long next()
        {
            if (this.occurrences == 0)
            {
                this.currentItem = this.longIterator.next();
                this.occurrences = LongHashBag.this.occurrencesOf(this.currentItem);
            }
            this.occurrences--;
            this.canRemove = true;
            return this.currentItem;
        }

        public void remove()
        {
            if (!this.canRemove)
            {
                throw new IllegalStateException();
            }
            if (this.occurrences == 0)
            {
                this.longIterator.remove();
                LongHashBag.this.size--;
            }
            else
            {
                LongHashBag.this.remove(this.currentItem);
            }
            this.canRemove = false;
        }
    }
}
