/*
 * Copyright (c) 2015 Goldman Sachs.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * and Eclipse Distribution License v. 1.0 which accompany this distribution.
 * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
 * and the Eclipse Distribution License is available at
 * http://www.eclipse.org/org/documents/edl-v10.php.
 */

package org.eclipse.collections.impl.map.mutable.primitive;

import java.io.Serializable;

import org.eclipse.collections.api.BooleanIterable;
import org.eclipse.collections.api.CharIterable;
import org.eclipse.collections.api.LazyBooleanIterable;
import org.eclipse.collections.api.LazyCharIterable;
import org.eclipse.collections.api.RichIterable;
import org.eclipse.collections.api.bag.primitive.MutableBooleanBag;
import org.eclipse.collections.api.block.function.primitive.BooleanFunction;
import org.eclipse.collections.api.block.function.primitive.BooleanFunction0;
import org.eclipse.collections.api.block.function.primitive.BooleanToBooleanFunction;
import org.eclipse.collections.api.block.function.primitive.BooleanToObjectFunction;
import org.eclipse.collections.api.block.function.primitive.ObjectBooleanToObjectFunction;
import org.eclipse.collections.api.block.function.primitive.CharToBooleanFunction;
import org.eclipse.collections.api.block.predicate.primitive.BooleanPredicate;
import org.eclipse.collections.api.block.predicate.primitive.CharBooleanPredicate;
import org.eclipse.collections.api.block.procedure.primitive.BooleanProcedure;
import org.eclipse.collections.api.block.procedure.primitive.CharBooleanProcedure;
import org.eclipse.collections.api.block.procedure.primitive.CharProcedure;
import org.eclipse.collections.api.collection.MutableCollection;
import org.eclipse.collections.api.collection.primitive.MutableBooleanCollection;
import org.eclipse.collections.api.iterator.MutableBooleanIterator;
import org.eclipse.collections.api.list.primitive.MutableBooleanList;
import org.eclipse.collections.api.map.primitive.ImmutableCharBooleanMap;
import org.eclipse.collections.api.map.primitive.CharBooleanMap;
import org.eclipse.collections.api.map.primitive.MutableCharBooleanMap;
import org.eclipse.collections.api.set.primitive.MutableCharSet;
import org.eclipse.collections.api.set.primitive.MutableBooleanSet;
import org.eclipse.collections.api.tuple.primitive.CharBooleanPair;
import org.eclipse.collections.impl.factory.primitive.CharBooleanMaps;
import org.eclipse.collections.impl.iterator.UnmodifiableBooleanIterator;
import org.eclipse.collections.impl.set.mutable.primitive.UnmodifiableCharSet;
import org.eclipse.collections.impl.collection.mutable.primitive.UnmodifiableBooleanCollection;

/**
 * This file was automatically generated from template file unmodifiablePrimitivePrimitiveMap.stg.
 *
 * @since 3.1.
 */
public final class UnmodifiableCharBooleanMap
        implements MutableCharBooleanMap, Serializable
{
    private static final long serialVersionUID = 1L;
    private final MutableCharBooleanMap map;

    UnmodifiableCharBooleanMap(MutableCharBooleanMap map)
    {
        this.map = map;
    }

    public void clear()
    {
        throw new UnsupportedOperationException("Cannot call clear() on " + this.getClass().getSimpleName());
    }

    public void put(char key, boolean value)
    {
        throw new UnsupportedOperationException("Cannot call put() on " + this.getClass().getSimpleName());
    }

    public void putAll(CharBooleanMap map)
    {
        throw new UnsupportedOperationException("Cannot call putAll() on " + this.getClass().getSimpleName());
    }

    public void removeKey(char key)
    {
        throw new UnsupportedOperationException("Cannot call removeKey() on " + this.getClass().getSimpleName());
    }

    public void remove(char key)
    {
        throw new UnsupportedOperationException("Cannot call remove() on " + this.getClass().getSimpleName());
    }

    public boolean removeKeyIfAbsent(char key, boolean value)
    {
        if (this.map.containsKey(key))
        {
            throw new UnsupportedOperationException("Cannot call removeKey() on " + this.getClass().getSimpleName());
        }
        return value;
    }

    public boolean getIfAbsentPut(char key, boolean value)
    {
        return this.map.getIfAbsentPut(key, new BooleanFunction0()
        {
            public boolean value()
            {
                throw new UnsupportedOperationException();
            }
        });
    }

    public boolean getIfAbsentPut(char key, BooleanFunction0 function)
    {
        return this.map.getIfAbsentPut(key, new BooleanFunction0()
        {
            public boolean value()
            {
                throw new UnsupportedOperationException();
            }
        });
    }

    public boolean getIfAbsentPutWithKey(char key, CharToBooleanFunction function)
    {
        return this.map.getIfAbsentPut(key, new BooleanFunction0()
        {
            public boolean value()
            {
                throw new UnsupportedOperationException();
            }
        });
    }

    public <P> boolean getIfAbsentPutWith(char key, BooleanFunction<? super P> function, P parameter)
    {
        return this.map.getIfAbsentPut(key, new BooleanFunction0()
        {
            public boolean value()
            {
                throw new UnsupportedOperationException();
            }
        });
    }

    public boolean updateValue(char key, boolean initialValueIfAbsent, BooleanToBooleanFunction function)
    {
        throw new UnsupportedOperationException("Cannot call updateValue() on " + this.getClass().getSimpleName());
    }

    public boolean get(char key)
    {
        return this.map.get(key);
    }

    public boolean getIfAbsent(char key, boolean ifAbsent)
    {
        return this.map.getIfAbsent(key, ifAbsent);
    }

    public boolean getOrThrow(char key)
    {
        return this.map.getOrThrow(key);
    }

    public boolean containsKey(char key)
    {
        return this.map.containsKey(key);
    }

    public boolean containsValue(boolean value)
    {
        return this.map.containsValue(value);
    }

    public void forEachValue(BooleanProcedure procedure)
    {
        this.map.forEachValue(procedure);
    }

    public void forEachKey(CharProcedure procedure)
    {
        this.map.forEachKey(procedure);
    }

    public void forEachKeyValue(CharBooleanProcedure procedure)
    {
        this.map.forEachKeyValue(procedure);
    }

    public LazyCharIterable keysView()
    {
        return this.map.keysView();
    }

    public RichIterable<CharBooleanPair> keyValuesView()
    {
        return this.map.keyValuesView();
    }

    public MutableCharBooleanMap select(CharBooleanPredicate predicate)
    {
        return this.map.select(predicate);
    }

    public MutableCharBooleanMap reject(CharBooleanPredicate predicate)
    {
        return this.map.reject(predicate);
    }

    public MutableBooleanIterator booleanIterator()
    {
        return new UnmodifiableBooleanIterator(this.map.booleanIterator());
    }

    public void forEach(BooleanProcedure procedure)
    {
        this.each(procedure);
    }

    /**
     * @since 7.0.
     */
    public void each(BooleanProcedure procedure)
    {
        this.map.forEach(procedure);
    }

    public int count(BooleanPredicate predicate)
    {
        return this.map.count(predicate);
    }

    public boolean anySatisfy(BooleanPredicate predicate)
    {
        return this.map.anySatisfy(predicate);
    }

    public boolean allSatisfy(BooleanPredicate predicate)
    {
        return this.map.allSatisfy(predicate);
    }

    public boolean noneSatisfy(BooleanPredicate predicate)
    {
        return this.map.noneSatisfy(predicate);
    }

    public MutableBooleanCollection select(BooleanPredicate predicate)
    {
        return this.map.select(predicate);
    }

    public MutableBooleanCollection reject(BooleanPredicate predicate)
    {
        return this.map.reject(predicate);
    }

    public boolean detectIfNone(BooleanPredicate predicate, boolean ifNone)
    {
        return this.map.detectIfNone(predicate, ifNone);
    }

    public <V> MutableCollection<V> collect(BooleanToObjectFunction<? extends V> function)
    {
        return this.map.collect(function);
    }

    public boolean[] toArray()
    {
        return this.map.toArray();
    }

    public boolean contains(boolean value)
    {
        return this.map.contains(value);
    }

    public boolean containsAll(boolean... source)
    {
        return this.map.containsAll(source);
    }

    public boolean containsAll(BooleanIterable source)
    {
        return this.map.containsAll(source);
    }

    public MutableBooleanList toList()
    {
        return this.map.toList();
    }

    public MutableBooleanSet toSet()
    {
        return this.map.toSet();
    }

    public MutableBooleanBag toBag()
    {
        return this.map.toBag();
    }

    public LazyBooleanIterable asLazy()
    {
        return this.map.asLazy();
    }

    public MutableCharBooleanMap withKeyValue(char key, boolean value)
    {
        throw new UnsupportedOperationException("Cannot call withKeyValue() on " + this.getClass().getSimpleName());
    }

    public MutableCharBooleanMap withoutKey(char key)
    {
        throw new UnsupportedOperationException("Cannot call withoutKey() on " + this.getClass().getSimpleName());
    }

    public MutableCharBooleanMap withoutAllKeys(CharIterable keys)
    {
        throw new UnsupportedOperationException("Cannot call withoutAllKeys() on " + this.getClass().getSimpleName());
    }

    public MutableCharBooleanMap asUnmodifiable()
    {
        return this;
    }

    public MutableCharBooleanMap asSynchronized()
    {
        return new SynchronizedCharBooleanMap(this);
    }

    public ImmutableCharBooleanMap toImmutable()
    {
        return CharBooleanMaps.immutable.withAll(this);
    }

    public int size()
    {
        return this.map.size();
    }

    public boolean isEmpty()
    {
        return this.map.isEmpty();
    }

    public boolean notEmpty()
    {
        return this.map.notEmpty();
    }

    public MutableCharSet keySet()
    {
        return UnmodifiableCharSet.of(this.map.keySet());
    }

    public MutableBooleanCollection values()
    {
        return UnmodifiableBooleanCollection.of(this.map.values());
    }

    @Override
    public boolean equals(Object otherMap)
    {
        return this.map.equals(otherMap);
    }

    @Override
    public int hashCode()
    {
        return this.map.hashCode();
    }

    @Override
    public String toString()
    {
        return this.map.toString();
    }

    public String makeString()
    {
        return this.map.makeString();
    }

    public String makeString(String separator)
    {
        return this.map.makeString(separator);
    }

    public String makeString(String start, String separator, String end)
    {
        return this.map.makeString(start, separator, end);
    }

    public void appendString(Appendable appendable)
    {
        this.map.appendString(appendable);
    }

    public void appendString(Appendable appendable, String separator)
    {
        this.map.appendString(appendable, separator);
    }

    public void appendString(Appendable appendable, String start, String separator, String end)
    {
        this.map.appendString(appendable, start, separator, end);
    }

    public <T> T injectInto(T injectedValue, ObjectBooleanToObjectFunction<? super T, ? extends T> function)
    {
        return this.map.injectInto(injectedValue, function);
    }
}
