/*
 * Copyright (c) 2015 Goldman Sachs.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * and Eclipse Distribution License v. 1.0 which accompany this distribution.
 * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
 * and the Eclipse Distribution License is available at
 * http://www.eclipse.org/org/documents/edl-v10.php.
 */

package org.eclipse.collections.impl.map.mutable.primitive;

import java.io.Serializable;

import org.eclipse.collections.api.DoubleIterable;
import org.eclipse.collections.api.LongIterable;
import org.eclipse.collections.api.LazyDoubleIterable;
import org.eclipse.collections.api.LazyLongIterable;
import org.eclipse.collections.api.RichIterable;
import org.eclipse.collections.api.bag.primitive.MutableDoubleBag;
import org.eclipse.collections.api.block.function.primitive.DoubleFunction;
import org.eclipse.collections.api.block.function.primitive.DoubleFunction0;
import org.eclipse.collections.api.block.function.primitive.DoubleToDoubleFunction;
import org.eclipse.collections.api.block.function.primitive.DoubleToObjectFunction;
import org.eclipse.collections.api.block.function.primitive.ObjectDoubleToObjectFunction;
import org.eclipse.collections.api.block.function.primitive.LongToDoubleFunction;
import org.eclipse.collections.api.block.predicate.primitive.DoublePredicate;
import org.eclipse.collections.api.block.predicate.primitive.LongDoublePredicate;
import org.eclipse.collections.api.block.procedure.primitive.DoubleProcedure;
import org.eclipse.collections.api.block.procedure.primitive.LongDoubleProcedure;
import org.eclipse.collections.api.block.procedure.primitive.LongProcedure;
import org.eclipse.collections.api.collection.MutableCollection;
import org.eclipse.collections.api.collection.primitive.MutableDoubleCollection;
import org.eclipse.collections.api.iterator.MutableDoubleIterator;
import org.eclipse.collections.api.list.primitive.MutableDoubleList;
import org.eclipse.collections.api.map.primitive.ImmutableLongDoubleMap;
import org.eclipse.collections.api.map.primitive.LongDoubleMap;
import org.eclipse.collections.api.map.primitive.MutableLongDoubleMap;
import org.eclipse.collections.api.set.primitive.MutableLongSet;
import org.eclipse.collections.api.set.primitive.MutableDoubleSet;
import org.eclipse.collections.api.tuple.primitive.LongDoublePair;
import org.eclipse.collections.impl.factory.primitive.LongDoubleMaps;
import org.eclipse.collections.impl.iterator.UnmodifiableDoubleIterator;
import org.eclipse.collections.impl.set.mutable.primitive.UnmodifiableLongSet;
import org.eclipse.collections.impl.collection.mutable.primitive.UnmodifiableDoubleCollection;

/**
 * This file was automatically generated from template file unmodifiablePrimitivePrimitiveMap.stg.
 *
 * @since 3.1.
 */
public final class UnmodifiableLongDoubleMap
        implements MutableLongDoubleMap, Serializable
{
    private static final long serialVersionUID = 1L;
    private final MutableLongDoubleMap map;

    UnmodifiableLongDoubleMap(MutableLongDoubleMap map)
    {
        this.map = map;
    }

    public void clear()
    {
        throw new UnsupportedOperationException("Cannot call clear() on " + this.getClass().getSimpleName());
    }

    public void put(long key, double value)
    {
        throw new UnsupportedOperationException("Cannot call put() on " + this.getClass().getSimpleName());
    }

    public void putAll(LongDoubleMap map)
    {
        throw new UnsupportedOperationException("Cannot call putAll() on " + this.getClass().getSimpleName());
    }

    public void removeKey(long key)
    {
        throw new UnsupportedOperationException("Cannot call removeKey() on " + this.getClass().getSimpleName());
    }

    public void remove(long key)
    {
        throw new UnsupportedOperationException("Cannot call remove() on " + this.getClass().getSimpleName());
    }

    public double removeKeyIfAbsent(long key, double value)
    {
        if (this.map.containsKey(key))
        {
            throw new UnsupportedOperationException("Cannot call removeKey() on " + this.getClass().getSimpleName());
        }
        return value;
    }

    public double getIfAbsentPut(long key, double value)
    {
        return this.map.getIfAbsentPut(key, new DoubleFunction0()
        {
            public double value()
            {
                throw new UnsupportedOperationException();
            }
        });
    }

    public double getIfAbsentPut(long key, DoubleFunction0 function)
    {
        return this.map.getIfAbsentPut(key, new DoubleFunction0()
        {
            public double value()
            {
                throw new UnsupportedOperationException();
            }
        });
    }

    public double getIfAbsentPutWithKey(long key, LongToDoubleFunction function)
    {
        return this.map.getIfAbsentPut(key, new DoubleFunction0()
        {
            public double value()
            {
                throw new UnsupportedOperationException();
            }
        });
    }

    public <P> double getIfAbsentPutWith(long key, DoubleFunction<? super P> function, P parameter)
    {
        return this.map.getIfAbsentPut(key, new DoubleFunction0()
        {
            public double value()
            {
                throw new UnsupportedOperationException();
            }
        });
    }

    public double updateValue(long key, double initialValueIfAbsent, DoubleToDoubleFunction function)
    {
        throw new UnsupportedOperationException("Cannot call updateValue() on " + this.getClass().getSimpleName());
    }

    public double get(long key)
    {
        return this.map.get(key);
    }

    public double getIfAbsent(long key, double ifAbsent)
    {
        return this.map.getIfAbsent(key, ifAbsent);
    }

    public double getOrThrow(long key)
    {
        return this.map.getOrThrow(key);
    }

    public boolean containsKey(long key)
    {
        return this.map.containsKey(key);
    }

    public boolean containsValue(double value)
    {
        return this.map.containsValue(value);
    }

    public void forEachValue(DoubleProcedure procedure)
    {
        this.map.forEachValue(procedure);
    }

    public void forEachKey(LongProcedure procedure)
    {
        this.map.forEachKey(procedure);
    }

    public void forEachKeyValue(LongDoubleProcedure procedure)
    {
        this.map.forEachKeyValue(procedure);
    }

    public LazyLongIterable keysView()
    {
        return this.map.keysView();
    }

    public RichIterable<LongDoublePair> keyValuesView()
    {
        return this.map.keyValuesView();
    }

    public MutableLongDoubleMap select(LongDoublePredicate predicate)
    {
        return this.map.select(predicate);
    }

    public MutableLongDoubleMap reject(LongDoublePredicate predicate)
    {
        return this.map.reject(predicate);
    }

    public MutableDoubleIterator doubleIterator()
    {
        return new UnmodifiableDoubleIterator(this.map.doubleIterator());
    }

    public void forEach(DoubleProcedure procedure)
    {
        this.each(procedure);
    }

    /**
     * @since 7.0.
     */
    public void each(DoubleProcedure procedure)
    {
        this.map.forEach(procedure);
    }

    public int count(DoublePredicate predicate)
    {
        return this.map.count(predicate);
    }

    public boolean anySatisfy(DoublePredicate predicate)
    {
        return this.map.anySatisfy(predicate);
    }

    public boolean allSatisfy(DoublePredicate predicate)
    {
        return this.map.allSatisfy(predicate);
    }

    public boolean noneSatisfy(DoublePredicate predicate)
    {
        return this.map.noneSatisfy(predicate);
    }

    public MutableDoubleCollection select(DoublePredicate predicate)
    {
        return this.map.select(predicate);
    }

    public MutableDoubleCollection reject(DoublePredicate predicate)
    {
        return this.map.reject(predicate);
    }

    public double detectIfNone(DoublePredicate predicate, double ifNone)
    {
        return this.map.detectIfNone(predicate, ifNone);
    }

    public <V> MutableCollection<V> collect(DoubleToObjectFunction<? extends V> function)
    {
        return this.map.collect(function);
    }

    public double sum()
    {
        return this.map.sum();
    }

    public double max()
    {
        return this.map.max();
    }

    public double maxIfEmpty(double defaultValue)
    {
        return this.map.maxIfEmpty(defaultValue);
    }

    public double min()
    {
        return this.map.min();
    }

    public double minIfEmpty(double defaultValue)
    {
        return this.map.minIfEmpty(defaultValue);
    }

    public double average()
    {
        return this.map.average();
    }

    public double median()
    {
        return this.map.median();
    }

    public double addToValue(long key, double toBeAdded)
    {
        throw new UnsupportedOperationException("Cannot call addToValue() on " + this.getClass().getSimpleName());
    }

    public double[] toSortedArray()
    {
        return this.map.toSortedArray();
    }

    public MutableDoubleList toSortedList()
    {
        return this.map.toSortedList();
    }

    public double[] toArray()
    {
        return this.map.toArray();
    }

    public boolean contains(double value)
    {
        return this.map.contains(value);
    }

    public boolean containsAll(double... source)
    {
        return this.map.containsAll(source);
    }

    public boolean containsAll(DoubleIterable source)
    {
        return this.map.containsAll(source);
    }

    public MutableDoubleList toList()
    {
        return this.map.toList();
    }

    public MutableDoubleSet toSet()
    {
        return this.map.toSet();
    }

    public MutableDoubleBag toBag()
    {
        return this.map.toBag();
    }

    public LazyDoubleIterable asLazy()
    {
        return this.map.asLazy();
    }

    public MutableLongDoubleMap withKeyValue(long key, double value)
    {
        throw new UnsupportedOperationException("Cannot call withKeyValue() on " + this.getClass().getSimpleName());
    }

    public MutableLongDoubleMap withoutKey(long key)
    {
        throw new UnsupportedOperationException("Cannot call withoutKey() on " + this.getClass().getSimpleName());
    }

    public MutableLongDoubleMap withoutAllKeys(LongIterable keys)
    {
        throw new UnsupportedOperationException("Cannot call withoutAllKeys() on " + this.getClass().getSimpleName());
    }

    public MutableLongDoubleMap asUnmodifiable()
    {
        return this;
    }

    public MutableLongDoubleMap asSynchronized()
    {
        return new SynchronizedLongDoubleMap(this);
    }

    public ImmutableLongDoubleMap toImmutable()
    {
        return LongDoubleMaps.immutable.withAll(this);
    }

    public int size()
    {
        return this.map.size();
    }

    public boolean isEmpty()
    {
        return this.map.isEmpty();
    }

    public boolean notEmpty()
    {
        return this.map.notEmpty();
    }

    public MutableLongSet keySet()
    {
        return UnmodifiableLongSet.of(this.map.keySet());
    }

    public MutableDoubleCollection values()
    {
        return UnmodifiableDoubleCollection.of(this.map.values());
    }

    @Override
    public boolean equals(Object otherMap)
    {
        return this.map.equals(otherMap);
    }

    @Override
    public int hashCode()
    {
        return this.map.hashCode();
    }

    @Override
    public String toString()
    {
        return this.map.toString();
    }

    public String makeString()
    {
        return this.map.makeString();
    }

    public String makeString(String separator)
    {
        return this.map.makeString(separator);
    }

    public String makeString(String start, String separator, String end)
    {
        return this.map.makeString(start, separator, end);
    }

    public void appendString(Appendable appendable)
    {
        this.map.appendString(appendable);
    }

    public void appendString(Appendable appendable, String separator)
    {
        this.map.appendString(appendable, separator);
    }

    public void appendString(Appendable appendable, String start, String separator, String end)
    {
        this.map.appendString(appendable, start, separator, end);
    }

    public <T> T injectInto(T injectedValue, ObjectDoubleToObjectFunction<? super T, ? extends T> function)
    {
        return this.map.injectInto(injectedValue, function);
    }
}
