/*
 * Copyright (c) 2015 Goldman Sachs.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * and Eclipse Distribution License v. 1.0 which accompany this distribution.
 * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
 * and the Eclipse Distribution License is available at
 * http://www.eclipse.org/org/documents/edl-v10.php.
 */

package org.eclipse.collections.impl.stack.mutable.primitive;

import java.io.Serializable;

import org.eclipse.collections.api.IntIterable;
import org.eclipse.collections.api.LazyIntIterable;
import org.eclipse.collections.api.bag.primitive.MutableIntBag;
import org.eclipse.collections.api.block.function.primitive.IntToObjectFunction;
import org.eclipse.collections.api.block.function.primitive.ObjectIntToObjectFunction;
import org.eclipse.collections.api.block.predicate.primitive.IntPredicate;
import org.eclipse.collections.api.block.procedure.primitive.IntProcedure;
import org.eclipse.collections.api.iterator.IntIterator;
import org.eclipse.collections.api.list.primitive.IntList;
import org.eclipse.collections.api.list.primitive.MutableIntList;
import org.eclipse.collections.api.set.primitive.MutableIntSet;
import org.eclipse.collections.api.stack.MutableStack;
import org.eclipse.collections.api.stack.primitive.ImmutableIntStack;
import org.eclipse.collections.api.stack.primitive.MutableIntStack;
import org.eclipse.collections.impl.factory.primitive.IntStacks;
import org.eclipse.collections.impl.iterator.UnmodifiableIntIterator;
import org.eclipse.collections.impl.lazy.primitive.LazyIntIterableAdapter;
import net.jcip.annotations.NotThreadSafe;

/**
 * This file was automatically generated from template file unmodifiablePrimitiveStack.stg.
 *
 * @since 3.1.
 */
@NotThreadSafe
public final class UnmodifiableIntStack
        implements MutableIntStack, Serializable
{
    private static final long serialVersionUID = 1L;

    private final MutableIntStack stack;

    UnmodifiableIntStack(MutableIntStack stack)
    {
        this.stack = stack;
    }

    public void push(int item)
    {
        throw new UnsupportedOperationException("Cannot call push() on " + this.getClass().getSimpleName());
    }

    public int pop()
    {
        throw new UnsupportedOperationException("Cannot call pop() on " + this.getClass().getSimpleName());
    }

    public IntList pop(int count)
    {
        throw new UnsupportedOperationException("Cannot call pop() on " + this.getClass().getSimpleName());
    }

    public int peek()
    {
        return this.stack.peek();
    }

    public IntList peek(int count)
    {
        return this.stack.peek(count);
    }

    public int peekAt(int index)
    {
        return this.stack.peekAt(index);
    }

    public int size()
    {
        return this.stack.size();
    }

    public boolean isEmpty()
    {
        return this.stack.isEmpty();
    }

    public boolean notEmpty()
    {
        return this.stack.notEmpty();
    }

    public void clear()
    {
        throw new UnsupportedOperationException("Cannot call clear() on " + this.getClass().getSimpleName());
    }

    public boolean contains(int value)
    {
        return this.stack.contains(value);
    }

    public boolean containsAll(int... source)
    {
        return this.stack.containsAll(source);
    }

    public boolean containsAll(IntIterable source)
    {
        return this.stack.containsAll(source);
    }

    public IntIterator intIterator()
    {
        return new UnmodifiableIntIterator(this.stack.intIterator());
    }

    public void forEach(IntProcedure procedure)
    {
        this.each(procedure);
    }

    /**
     * @since 7.0.
     */
    public void each(IntProcedure procedure)
    {
        this.stack.forEach(procedure);
    }

    public int count(IntPredicate predicate)
    {
        return this.stack.count(predicate);
    }

    public boolean anySatisfy(IntPredicate predicate)
    {
        return this.stack.anySatisfy(predicate);
    }

    public boolean allSatisfy(IntPredicate predicate)
    {
        return this.stack.allSatisfy(predicate);
    }

    public boolean noneSatisfy(IntPredicate predicate)
    {
        return this.stack.noneSatisfy(predicate);
    }

    public int detectIfNone(IntPredicate predicate, int ifNone)
    {
        return this.stack.detectIfNone(predicate, ifNone);
    }

    public MutableIntStack select(IntPredicate predicate)
    {
        return this.stack.select(predicate);
    }

    public MutableIntStack reject(IntPredicate predicate)
    {
        return this.stack.reject(predicate);
    }

    public <V> MutableStack<V> collect(IntToObjectFunction<? extends V> function)
    {
        return this.stack.collect(function);
    }

    public long sum()
    {
        return this.stack.sum();
    }

    public int max()
    {
        return this.stack.max();
    }

    public int min()
    {
        return this.stack.min();
    }

    public int minIfEmpty(int defaultValue)
    {
        return this.stack.minIfEmpty(defaultValue);
    }

    public int maxIfEmpty(int defaultValue)
    {
        return this.stack.maxIfEmpty(defaultValue);
    }

    public double average()
    {
        return this.stack.average();
    }

    public double median()
    {
        return this.stack.median();
    }

    public MutableIntList toSortedList()
    {
        return this.stack.toSortedList();
    }

    public int[] toSortedArray()
    {
        return this.stack.toSortedArray();
    }

    public int[] toArray()
    {
        return this.stack.toArray();
    }

    @Override
    public String toString()
    {
        return this.stack.toString();
    }

    public String makeString()
    {
        return this.stack.makeString();
    }

    public String makeString(String separator)
    {
        return this.stack.makeString(separator);
    }

    public String makeString(String start, String separator, String end)
    {
        return this.stack.makeString(start, separator, end);
    }

    public void appendString(Appendable appendable)
    {
        this.stack.appendString(appendable);
    }

    public void appendString(Appendable appendable, String separator)
    {
        this.stack.appendString(appendable, separator);
    }

    public void appendString(
            Appendable appendable,
            String start,
            String separator,
            String end)
    {
        this.stack.appendString(appendable, start, separator, end);
    }

    public MutableIntList toList()
    {
        return this.stack.toList();
    }

    public MutableIntSet toSet()
    {
        return this.stack.toSet();
    }

    public MutableIntBag toBag()
    {
        return this.stack.toBag();
    }

    @Override
    public boolean equals(Object otherStack)
    {
        return this.stack.equals(otherStack);
    }

    @Override
    public int hashCode()
    {
        return this.stack.hashCode();
    }

    public LazyIntIterable asLazy()
    {
        return new LazyIntIterableAdapter(this);
    }

    public MutableIntStack asUnmodifiable()
    {
        return this;
    }

    public MutableIntStack asSynchronized()
    {
        return new SynchronizedIntStack(this);
    }

    public ImmutableIntStack toImmutable()
    {
        return IntStacks.immutable.withAllReversed(this);
    }

    public <T> T injectInto(T injectedValue, ObjectIntToObjectFunction<? super T, ? extends T> function)
    {
        return this.stack.injectInto(injectedValue, function);
    }
}
